@@ -26,8 +26,13 @@
 #include <atomic>
 #include <mutex>
 #include <condition_variable>
+#if FCGI_IN_SUBDIR
 #include <fastcgi/fcgiapp.h>
 #include <fastcgi/fcgio.h>
+#else
+#include <fcgiapp.h>
+#include <fcgio.h>
+#endif
 #include <iostream>
 #include <fstream>
 
@@ -281,7 +286,7 @@ public:
 
         FCGX_Request request;
         FCGX_InitRequest(&request, sockFD,0);
-        AutoDispose<FCGX_Request*> guard( &request, boost::bind( &FCGX_Free, _1, 0 ) );
+        AutoDispose<FCGX_Request*> guard( &request, std::bind( &FCGX_Free, _1, 0 ) );
 
         struct pollfd fds[] { {
             _wakeupPipe[0],
