@@ -14,11 +14,10 @@ namespace std {
 
 #endif
 
-
-#if __cplusplus < 201703L
-
 namespace std {
 
+#if __cplusplus < 202002L
+
 //implementation of the detector idiom, used to help with SFINAE
 //from https://en.cppreference.com/w/cpp/experimental/is_detected
 
@@ -73,6 +72,18 @@ using is_detected_convertible = std::is_convertible<detected_t<Op, Args...>, To>
 template <class To, template<class...> class Op, class... Args>
 constexpr bool is_detected_convertible_v = is_detected_convertible<To, Op, Args...>::value_t::value;
 
+template< class T >
+struct remove_cvref {
+  using type = std::remove_cv_t<std::remove_reference_t<T>>;
+};
+
+template< class T >
+using remove_cvref_t = typename remove_cvref<T>::type;
+
+#endif
+
+#if __cplusplus < 201703L
+
 //https://en.cppreference.com/w/cpp/types/conjunction)
 template<class...> struct conjunction : std::true_type { };
 template<class B1> struct conjunction<B1> : B1 { };
@@ -91,8 +102,8 @@ struct disjunction<B1, Bn...>
 template<class B>
 struct negation : std::bool_constant< !bool(B::value)> { };
 
-}
 #endif
+}
 
 
 namespace zyppng {
@@ -105,19 +116,19 @@ namespace zyppng {
 
   //Provides the member typedef type which is the type pointed to by T, or, if T is not a pointer, then type is the same as T.
   template< typename T>
-  struct remove_smart_ptr{ using type = typename std::remove_cv<T>::type; };
+  struct remove_smart_ptr{ using type = std::remove_cv_t<T>; };
 
   template< typename T>
-  struct remove_smart_ptr<std::shared_ptr<T>>{ using type = typename std::remove_cv<T>::type; };
+  struct remove_smart_ptr<std::shared_ptr<T>>{ using type = std::remove_cv_t<T>; };
 
   template< typename T>
-  struct remove_smart_ptr<std::unique_ptr<T>>{ using type = typename std::remove_cv<T>::type; };
+  struct remove_smart_ptr<std::unique_ptr<T>>{ using type = std::remove_cv_t<T>; };
 
   template< typename T>
   using remove_smart_ptr_t = typename remove_smart_ptr<T>::type;
 
   //helper template to print type parameters
-  template <typename T>
+  template <typename ...T>
   class myerror_t;
 }
 
