@@ -6,7 +6,7 @@
 |                         /_____||_| |_| |_|                           |
 |                                                                      |
 ----------------------------------------------------------------------*/
-#include <zypp/APIConfig.h>
+#include <zypp-core/Globals.h>
 #include <zypp-curl/ng/network/private/networkrequestdispatcher_p.h>
 #include <zypp-curl/ng/network/private/networkrequesterror_p.h>
 #include <zypp-curl/ng/network/private/request_p.h>
@@ -329,6 +329,9 @@ bool NetworkRequestDispatcherPrivate::addRequestToMultiHandle(NetworkRequest &re
     setFinished( req, NetworkRequestErrorPrivate::fromCurlMError( rc ) );
     return false;
   }
+
+  // make sure to wake up once to register what we have now
+  _timer->start(0);
   return true;
 }
 
