@@ -42,7 +42,7 @@ namespace zyppng {
     ZYPP_DECLARE_PUBLIC(Download)
   public:
     DownloadPrivateBase ( Downloader &parent, std::shared_ptr<NetworkRequestDispatcher> requestDispatcher, std::shared_ptr<MirrorControl> mirrors, DownloadSpec &&spec, Download &p );
-    ~DownloadPrivateBase ();
+    ~DownloadPrivateBase () override;
 
     struct Block {
       off_t  start = 0;
@@ -51,6 +51,7 @@ namespace zyppng {
       std::string chksumtype;
       std::optional<UByteArray> chksumVec;
       std::optional<size_t> chksumCompareLen; //< initialized if only the first few bytes of the checksum should be considered
+      std::optional<size_t> chksumPad; //< initialized if the hashed blocks for a digest need to be padded if a block is smaller ( e.g. last block in a zsync file )
 
       int _retryCount = 0;  //< how many times was this request restarted
       NetworkRequestError _failedWithErr; //< what was the error this request failed with
@@ -81,7 +82,7 @@ namespace zyppng {
 
 
     bool _emittedSigStart = false;
-    bool handleRequestAuthError(std::shared_ptr<Request> req, const zyppng::NetworkRequestError &err);
+    bool handleRequestAuthError(const std::shared_ptr<Request>& req, const zyppng::NetworkRequestError &err);
 
     NetworkRequestError safeFillSettingsFromURL ( const Url &url, TransferSettings &set );
 
