@@ -389,6 +389,13 @@ void MediaCurl::setCookieFile( const Pathname &fileName )
   _cookieFile = fileName;
 }
 
+void MediaCurl::setCurlError(const char* error)
+{
+  // FIXME(dmllr): Use strlcpy if available for better performance
+  strncpy(_curlError, error, sizeof(_curlError)-1);
+  _curlError[sizeof(_curlError)-1] = '\0';
+}
+
 ///////////////////////////////////////////////////////////////////
 
 void MediaCurl::checkProtocol(const Url &url) const
@@ -1440,6 +1447,15 @@ CURLcode MediaCurl::executeCurl() const
   bool canContinue = true;
   while ( canContinue ) {
 
+    CURLMsg *msg = nullptr;
+    int nqueue = 0;
+    while ((msg = curl_multi_info_read( cMulti->_multi, &nqueue)) != 0) {
+        if ( msg->msg != CURLMSG_DONE  ) continue;
+        if ( msg->easy_handle != _curl ) continue;
+
+        return msg->data.result;
+    }
+
     // copy watched sockets in case curl changes the vector as we go over the events later
     std::vector<GPollFD> requestedFds = _curlHelper.socks;
 
@@ -1457,15 +1473,6 @@ CURLcode MediaCurl::executeCurl() const
       if (mcode != CURLM_OK)
         ZYPP_THROW(MediaCurlException(_url, "curl_multi_socket_action", "unknown error"));
     }
-
-    CURLMsg *msg = nullptr;
-    int nqueue = 0;
-    while ((msg = curl_multi_info_read( cMulti->_multi, &nqueue)) != 0) {
-        if ( msg->msg != CURLMSG_DONE  ) continue;
-        if ( msg->easy_handle != _curl ) continue;
-
-        return msg->data.result;
-    }
   }
   return CURLE_OK;
 }
