@@ -51,7 +51,7 @@ namespace zypp
     /////////////////////////////////////////////////////////////////
 
     /** lsof output line + files extracted so far for this PID */
-    typedef std::pair<std::string,std::unordered_set<std::string>> CacheEntry;
+    using CacheEntry = std::pair<std::string, std::unordered_set<std::string>>;
 
     /////////////////////////////////////////////////////////////////
     /// \class FilterRunsInContainer
@@ -89,6 +89,15 @@ namespace zypp
         const Pathname linkTarget = filesystem::readlink( path );
         if ( linkTarget.empty() ) return IGNORE;
 
+        // Pipe or socket 'type:[inode]' or an 'anon_inode:<file-type>'
+        // They may or may not belong to a container... (bsc#1218291)
+        if ( linkTarget.relative() ) return IGNORE;
+
+        // bsc#1226014. Ignore snaps. Execuables below /snap/
+        // (may also be detectable via /proc/PID/cgroup)
+        if ( str::startsWith( linkTarget.asString(), "/snap/" ) )
+          return CONTAINER;
+
         // get stat info for the target file
         const PathInfo linkStat( linkTarget );
 
@@ -172,19 +181,7 @@ namespace zypp
     bool lsofNoOptKi()
     {
       using target::rpm::librpmDb;
-      // RpmDb access is blocked while the Target is not initialized.
-      // Launching the Target just for this query would be an overkill.
-      struct TmpUnblock {
-        TmpUnblock()
-        : _wasBlocked( librpmDb::isBlocked() )
-        { if ( _wasBlocked ) librpmDb::unblockAccess(); }
-        ~TmpUnblock()
-        { if ( _wasBlocked ) librpmDb::blockAccess(); }
-      private:
-        bool _wasBlocked;
-      } tmpUnblock;
-
-      librpmDb::db_const_iterator it;
+      librpmDb::db_const_iterator it( "/" );
       return( it.findPackage( "lsof" ) && it->tag_edition() < Edition("4.90") && !it->tag_provides().count( Capability("backported-option-Ki") ) );
     }
 
@@ -351,6 +348,7 @@ namespace zypp
         , "/tmp/"
         , "/proc/"
         , "/memfd:"
+        , "/snap/"
       };
       for_( it, arrayBegin( black ), arrayEnd( black ) )
       {
@@ -532,11 +530,15 @@ namespace zypp
 
   std::string CheckAccessDeleted::ProcInfo::service() const
   {
-    static const str::regex rx( "(0::|[0-9]+:name=systemd:)/system.slice/(.*/)?(.*).service$" );
+    // cgroup entries like:
+    // 1:name=systemd:/system.slice/systemd-udevd.service
+    // 0::/system.slice/systemd-udevd.service
+    // 0::/system.slice/systemd-udevd.service/udev
+    static const str::regex rx( "(0::|[0-9]+:name=systemd:)/system.slice/(.*/)?(.*).service(/.*)?$" );
     str::smatch what;
     std::string ret;
     iostr::simpleParseFile( InputStream( Pathname("/proc")/pid/"cgroup" ),
-                            [&]( int num_r, std::string line_r )->bool
+                            [&]( int num_r, const std::string& line_r )->bool
                             {
                               if ( str::regex_match( line_r, what, rx ) )
                               {
