@@ -17,6 +17,7 @@
 #include <zypp/base/Gettext.h>
 #include <zypp/base/Logger.h>
 #include <zypp/base/String.h>
+#include <utility>
 #include <zypp-core/base/UserRequestException>
 #include <zypp/repo/RepoProvideFile.h>
 #include <zypp/ZYppCallbacks.h>
@@ -58,17 +59,24 @@ namespace zypp
       */
       struct DownloadFileReportHack : public callback::ReceiveReport<media::DownloadProgressReport>
       {
-        typedef callback::ReceiveReport<ReportType> BaseType;
-        typedef function<bool(int)>                 RedirectType;
+        using BaseType = callback::ReceiveReport<ReportType>;
+        using RedirectType = function<bool (int)>;
 
-        DownloadFileReportHack( RedirectType redirect_r )
-        : _oldRec( Distributor::instance().getReceiver() )
-        , _redirect( redirect_r )
-        { connect(); }
-        ~DownloadFileReportHack()
+        DownloadFileReportHack(RedirectType &&redirect_r)
+          : _oldRec(Distributor::instance().getReceiver()),
+            _redirect(std::move(redirect_r)) {
+          connect();
+        }
+
+        DownloadFileReportHack(const DownloadFileReportHack &) = delete;
+        DownloadFileReportHack(DownloadFileReportHack &&) = delete;
+        DownloadFileReportHack &operator=(const DownloadFileReportHack &) = delete;
+        DownloadFileReportHack &operator=(DownloadFileReportHack &&) = delete;
+
+        ~DownloadFileReportHack() override
         { if ( _oldRec ) Distributor::instance().setReceiver( *_oldRec ); else Distributor::instance().noReceiver(); }
 
-        virtual void start( const Url & file, Pathname localfile )
+        void start( const Url & file, Pathname localfile ) override
         {
           if ( _oldRec )
             _oldRec->start( file, localfile );
@@ -76,7 +84,7 @@ namespace zypp
             BaseType::start( file, localfile );
         }
 
-        virtual bool progress( int value, const Url & file, double dbps_avg = -1, double dbps_current = -1 )
+        bool progress( int value, const Url & file, double dbps_avg = -1, double dbps_current = -1 ) override
         {
           bool ret = true;
           if ( _oldRec )
@@ -86,13 +94,13 @@ namespace zypp
           return ret;
         }
 
-        virtual Action problem( const Url & file, Error error, const std::string & description )
+        Action problem( const Url & file, Error error, const std::string & description ) override
         {
           if ( _oldRec )
             return _oldRec->problem( file, error, description );
           return BaseType::problem( file, error, description );
         }
-        virtual void finish( const Url & file, Error error, const std::string & reason )
+        void finish( const Url & file, Error error, const std::string & reason ) override
         {
           if ( _oldRec )
             _oldRec->finish( file, error, reason );
@@ -114,16 +122,20 @@ namespace zypp
                              const ProvideFilePolicy & policy_r )
     {
       RepoMediaAccess access;
-      return access.provideFile(repo_r, loc_r, policy_r );
+      return access.provideFile(std::move(repo_r), loc_r, policy_r );
     }
 
     ///////////////////////////////////////////////////////////////////
     class RepoMediaAccess::Impl
     {
     public:
-      Impl( const ProvideFilePolicy & defaultPolicy_r )
-        : _defaultPolicy( defaultPolicy_r )
-      {}
+      Impl(ProvideFilePolicy &&defaultPolicy_r)
+        : _defaultPolicy(std::move(defaultPolicy_r)) {}
+
+      Impl(const Impl &) = delete;
+      Impl(Impl &&) = delete;
+      Impl &operator=(const Impl &) = delete;
+      Impl &operator=(Impl &&) = delete;
 
       ~Impl()
       {
@@ -157,12 +169,12 @@ namespace zypp
           media.reset( new MediaSetAccess(url) );
           _medias[url] = media;
         }
-        setVerifierForRepo( repo, media );
+        setVerifierForRepo( std::move(repo), media );
         return media;
       }
 
       private:
-        void setVerifierForRepo( RepoInfo repo, shared_ptr<MediaSetAccess> media )
+        void setVerifierForRepo( const RepoInfo& repo, const shared_ptr<MediaSetAccess>& media )
         {
           // Always set the MediaSetAccess label.
           media->setLabel( repo.name() );
@@ -219,8 +231,8 @@ namespace zypp
     ///////////////////////////////////////////////////////////////////
 
 
-    RepoMediaAccess::RepoMediaAccess( const ProvideFilePolicy & defaultPolicy_r )
-      : _impl( new Impl( defaultPolicy_r ) )
+    RepoMediaAccess::RepoMediaAccess( ProvideFilePolicy defaultPolicy_r )
+      : _impl( new Impl( std::move(defaultPolicy_r) ) )
     {}
 
     RepoMediaAccess::~RepoMediaAccess()
@@ -232,7 +244,7 @@ namespace zypp
     const ProvideFilePolicy & RepoMediaAccess::defaultPolicy() const
     { return _impl->_defaultPolicy; }
 
-    ManagedFile RepoMediaAccess::provideFile( RepoInfo repo_r,
+    ManagedFile RepoMediaAccess::provideFile( const RepoInfo& repo_r,
                                               const OnMediaLocation & loc_rx,
                                               const ProvideFilePolicy & policy_r )
     {
@@ -242,7 +254,7 @@ namespace zypp
       // Arrange DownloadFileReportHack to recieve the source::DownloadFileReport
       // and redirect download progress triggers to call the ProvideFilePolicy
       // callback.
-      DownloadFileReportHack dumb( bind( mem_fun_ref( &ProvideFilePolicy::progress ), ref( policy_r ), _1 ) );
+      DownloadFileReportHack dumb( std::bind( std::mem_fn(&ProvideFilePolicy::progress), std::ref(policy_r), _1 ) );
 
       RepoException repo_excpt(repo_r,
                                str::form(_("Can't provide file '%s' from repository '%s'"),
