@@ -26,6 +26,7 @@ extern "C"
 #include <map>
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 #include <algorithm>
 
@@ -44,6 +45,7 @@ extern "C"
 
 #include <zypp/target/rpm/RpmDb.h>
 #include <zypp/target/rpm/RpmCallbacks.h>
+#include <zypp/target/RpmPostTransCollector.h>
 
 #include <zypp/HistoryLog.h>
 #include <zypp/target/rpm/librpmDb.h>
@@ -64,6 +66,10 @@ using namespace zypp::filesystem;
 
 #define WORKAROUNDRPMPWDBUG
 
+// bsc#1216091 indicates that 'rpm --runposttrans' does not execute the
+// scripts chroot if --root is used. We disable it if --root is not /.
+#define WORKAROUNDDUMPPOSTTRANSBUG
+
 #undef ZYPP_BASE_LOGGER_LOGGROUP
 #define ZYPP_BASE_LOGGER_LOGGROUP "librpmDb"
 
@@ -93,11 +99,7 @@ namespace rpm
 
 namespace
 {
-#if 1 // No more need to escape whitespace since rpm-4.4.2.3
 const char* quoteInFilename_m = "\'\"";
-#else
-const char* quoteInFilename_m = " \t\'\"";
-#endif
 inline std::string rpmQuoteFilename( const Pathname & path_r )
 {
   std::string path( path_r.asString() );
@@ -134,23 +136,24 @@ inline std::string rpmQuoteFilename( const Pathname & path_r )
 
 struct KeyRingSignalReceiver : callback::ReceiveReport<KeyRingSignals>
 {
-  KeyRingSignalReceiver(RpmDb &rpmdb) : _rpmdb(rpmdb)
-  {
-    connect();
-  }
+  KeyRingSignalReceiver(RpmDb &rpmdb) : _rpmdb(rpmdb) { connect(); }
+  KeyRingSignalReceiver(const KeyRingSignalReceiver &) = delete;
+  KeyRingSignalReceiver(KeyRingSignalReceiver &&) = delete;
+  KeyRingSignalReceiver &operator=(const KeyRingSignalReceiver &) = delete;
+  KeyRingSignalReceiver &operator=(KeyRingSignalReceiver &&) = delete;
 
-  ~KeyRingSignalReceiver()
+  ~KeyRingSignalReceiver() override
   {
     disconnect();
   }
 
-  virtual void trustedKeyAdded( const PublicKey &key )
+  void trustedKeyAdded( const PublicKey &key ) override
   {
     MIL << "trusted key added to zypp Keyring. Importing..." << endl;
     _rpmdb.importPubkey( key );
   }
 
-  virtual void trustedKeyRemoved( const PublicKey &key  )
+  void trustedKeyRemoved( const PublicKey &key  ) override
   {
     MIL << "Trusted key removed from zypp Keyring. Removing..." << endl;
     _rpmdb.removePubkey( key );
@@ -161,7 +164,7 @@ struct KeyRingSignalReceiver : callback::ReceiveReport<KeyRingSignals>
 
 static shared_ptr<KeyRingSignalReceiver> sKeyRingReceiver;
 
-unsigned diffFiles(const std::string file1, const std::string file2, std::string& out, int maxlines)
+unsigned diffFiles(const std::string& file1, const std::string& file2, std::string& out, int maxlines)
 {
   const char* argv[] =
     {
@@ -189,19 +192,6 @@ unsigned diffFiles(const std::string file1, const std::string file2, std::string
   return prog.close();
 }
 
-
-
-/******************************************************************
- **
- **
- **	FUNCTION NAME : stringPath
- **	FUNCTION TYPE : inline std::string
-*/
-inline std::string stringPath( const Pathname & root_r, const Pathname & sub_r )
-{
-  return librpmDb::stringPath( root_r, sub_r );
-}
-
 ///////////////////////////////////////////////////////////////////
 //
 //	CLASS NAME : RpmDb
@@ -254,7 +244,15 @@ RpmDb::~RpmDb()
 //
 std::ostream & RpmDb::dumpOn( std::ostream & str ) const
 {
-  return str << "RpmDb[" << stringPath( _root, _dbPath ) << "]";
+  return str << "RpmDb[" << dumpPath( _root, _dbPath ) << "]";
+}
+
+///////////////////////////////////////////////////////////////////
+RpmDb::db_const_iterator RpmDb::dbConstIterator() const
+{
+  if ( initialized() )
+    return db_const_iterator( root(), dbPath() );
+  return db_const_iterator();
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -268,8 +266,6 @@ void RpmDb::initDatabase( Pathname root_r, bool doRebuild_r )
   ///////////////////////////////////////////////////////////////////
   // Check arguments
   ///////////////////////////////////////////////////////////////////
-  bool quickinit( root_r.empty() );
-
   if ( root_r.empty() )
     root_r = "/";
 
@@ -295,40 +291,22 @@ void RpmDb::initDatabase( Pathname root_r, bool doRebuild_r )
     // to /usr/lib/sysimage/rpm. We continue to use the old dbpath
     // (via the compat symlink) until a re-init.
     if ( root_r == _root ) {
-      MIL << "Calling initDatabase: already initialized at " << stringPath( _root, _dbPath ) << endl;
+      MIL << "Calling initDatabase: already initialized at " << dumpPath( _root, _dbPath ) << endl;
       return;
     }
     else
       ZYPP_THROW(RpmDbAlreadyOpenException(_root, _dbPath, root_r, dbPath_r));
   }
 
-  MIL << "Calling initDatabase: " << stringPath( root_r, dbPath_r )
-      << ( doRebuild_r ? " (rebuilddb)" : "" )
-      << ( quickinit ? " (quickinit)" : "" ) << endl;
+  MIL << "Calling initDatabase: " << dumpPath( root_r, dbPath_r )
+      << ( doRebuild_r ? " (rebuilddb)" : "" ) << endl;
 
   ///////////////////////////////////////////////////////////////////
   // init database
   ///////////////////////////////////////////////////////////////////
-  librpmDb::unblockAccess();
-
-  if ( quickinit )
-  {
-    MIL << "QUICK initDatabase (no systemRoot set)" << endl;
-    return;
-  }
-
-  try
-  {
-    // creates dbdir and empty rpm database if not present
-    librpmDb::dbAccess( root_r );
-  }
-  catch (const RpmException & excpt_r)
-  {
-    ZYPP_CAUGHT(excpt_r);
-    librpmDb::blockAccess();
-    ZYPP_RETHROW(excpt_r);
-  }
-
+  // creates dbdir and empty rpm database if not present
+  // or throws RpmException
+  librpmDb::dbOpenCreate( root_r, dbPath_r );
   _root   = root_r;
   _dbPath = dbPath_r;
 
@@ -338,12 +316,13 @@ void RpmDb::initDatabase( Pathname root_r, bool doRebuild_r )
   MIL << "Synchronizing keys with zypp keyring" << endl;
   syncTrustedKeys();
 
+#if 0 // if this is needed we need to forcefully close the db of running db_const_iterators
   // Close the database in case any write acces (create/convert)
   // happened during init. This should drop any lock acquired
   // by librpm. On demand it will be reopened readonly and should
   // not hold any lock.
   librpmDb::dbRelease( true );
-
+#endif
   MIL << "InitDatabase: " << *this << endl;
 }
 
@@ -360,19 +339,11 @@ void RpmDb::closeDatabase()
     return;
   }
 
-  MIL << "Calling closeDatabase: " << *this << endl;
-
-  ///////////////////////////////////////////////////////////////////
-  // Block further database access
-  ///////////////////////////////////////////////////////////////////
-  librpmDb::blockAccess();
-
-  ///////////////////////////////////////////////////////////////////
-  // Uninit
-  ///////////////////////////////////////////////////////////////////
-  _root = _dbPath = Pathname();
-
+  // NOTE: There are no persistent librpmDb handles to invalidate.
+  // Running db_const_iterator may keep the DB physically open until they
+  // go out of scope too.
   MIL << "closeDatabase: " << *this << endl;
+  _root = _dbPath = Pathname();
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -425,7 +396,7 @@ void RpmDb::doRebuildDatabase(callback::SendReport<RebuildDBReport> & report)
   ProgressData tics;
   {
     ProgressData::value_type hdrTotal = 0;
-    for ( librpmDb::db_const_iterator it; *it; ++it, ++hdrTotal )
+    for ( auto it = dbConstIterator(); *it; ++it, ++hdrTotal )
     {;}
     tics.range( hdrTotal );
   }
@@ -624,7 +595,7 @@ void RpmDb::syncTrustedKeys( SyncTrustedKeyBits mode_r )
     MIL << "Exporting rpm keyring into zypp trusted keyring" <<endl;
     // Temporarily disconnect to prevent the attempt to re-import the exported keys.
     callback::TempConnect<KeyRingSignals> tempDisconnect;
-    librpmDb::db_const_iterator keepDbOpen; // just to keep a ref.
+    auto keepDbOpen = dbConstIterator(); // just to keep a ref.
 
     TmpFile tmpfile( getZYpp()->tmpPath() );
     {
@@ -792,7 +763,7 @@ void RpmDb::importPubkey( const PublicKey & pubkey_r )
     RpmSubprocessException excp( str::Format(_("Failed to import public key %1%") ) % pubkey_r.asString() );
     excp.moveToHistory( excplines );
     excp.addHistory( std::move(error_message) );
-    ZYPP_THROW( std::move(excp) );
+    ZYPP_THROW( excp );
   }
   else
   {
@@ -859,7 +830,7 @@ void RpmDb::removePubkey( const PublicKey & pubkey_r )
     RpmSubprocessException excp( str::Format(_("Failed to remove public key %1%") ) % pubkey_r.asString() );
     excp.moveToHistory( excplines );
     excp.addHistory( std::move(error_message) );
-    ZYPP_THROW( std::move(excp) );
+    ZYPP_THROW( excp );
   }
   else
   {
@@ -877,7 +848,7 @@ std::list<PublicKey> RpmDb::pubkeys() const
 {
   std::list<PublicKey> ret;
 
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   for ( it.findByName( "gpg-pubkey" ); *it; ++it )
   {
     Edition edition = it->tag_edition();
@@ -915,7 +886,7 @@ std::set<Edition> RpmDb::pubkeyEditions() const
   {
     std::set<Edition> ret;
 
-    librpmDb::db_const_iterator it;
+    auto it = dbConstIterator();
     for ( it.findByName( "gpg-pubkey" ); *it; ++it )
     {
       Edition edition = it->tag_edition();
@@ -939,8 +910,8 @@ RpmDb::fileList( const std::string & name_r, const Edition & edition_r ) const
 {
   std::list<FileInfo> result;
 
-  librpmDb::db_const_iterator it;
-  bool found;
+  auto it = dbConstIterator();
+  bool found = false;
   if (edition_r == Edition::noedition)
   {
     found = it.findPackage( name_r );
@@ -966,8 +937,8 @@ RpmDb::fileList( const std::string & name_r, const Edition & edition_r ) const
 //
 bool RpmDb::hasFile( const std::string & file_r, const std::string & name_r ) const
 {
-  librpmDb::db_const_iterator it;
-  bool res;
+  auto it = dbConstIterator();
+  bool res = false;
   do
   {
     res = it.findByFile( file_r );
@@ -992,7 +963,7 @@ bool RpmDb::hasFile( const std::string & file_r, const std::string & name_r ) co
 //
 std::string RpmDb::whoOwnsFile( const std::string & file_r) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   if (it.findByFile( file_r ))
   {
     return it->tag_name();
@@ -1010,7 +981,7 @@ std::string RpmDb::whoOwnsFile( const std::string & file_r) const
 //
 bool RpmDb::hasProvides( const std::string & tag_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   return it.findByProvides( tag_r );
 }
 
@@ -1024,7 +995,7 @@ bool RpmDb::hasProvides( const std::string & tag_r ) const
 //
 bool RpmDb::hasRequiredBy( const std::string & tag_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   return it.findByRequiredBy( tag_r );
 }
 
@@ -1038,7 +1009,7 @@ bool RpmDb::hasRequiredBy( const std::string & tag_r ) const
 //
 bool RpmDb::hasConflicts( const std::string & tag_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   return it.findByConflicts( tag_r );
 }
 
@@ -1052,7 +1023,7 @@ bool RpmDb::hasConflicts( const std::string & tag_r ) const
 //
 bool RpmDb::hasPackage( const std::string & name_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   return it.findPackage( name_r );
 }
 
@@ -1066,7 +1037,7 @@ bool RpmDb::hasPackage( const std::string & name_r ) const
 //
 bool RpmDb::hasPackage( const std::string & name_r, const Edition & ed_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   return it.findPackage( name_r, ed_r );
 }
 
@@ -1081,11 +1052,13 @@ bool RpmDb::hasPackage( const std::string & name_r, const Edition & ed_r ) const
 void RpmDb::getData( const std::string & name_r,
                      RpmHeader::constPtr & result_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   it.findPackage( name_r );
   result_r = *it;
+#if 0 // if this is needed we need to forcefully close the db of running db_const_iterators
   if (it.dbError())
     ZYPP_THROW(*(it.dbError()));
+#endif
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -1099,11 +1072,13 @@ void RpmDb::getData( const std::string & name_r,
 void RpmDb::getData( const std::string & name_r, const Edition & ed_r,
                      RpmHeader::constPtr & result_r ) const
 {
-  librpmDb::db_const_iterator it;
+  auto it = dbConstIterator();
   it.findPackage( name_r, ed_r  );
   result_r = *it;
+#if 0 // if this is needed we need to forcefully close the db of running db_const_iterators
   if (it.dbError())
     ZYPP_THROW(*(it.dbError()));
+#endif
 }
 
 ///////////////////////////////////////////////////////////////////
@@ -1117,8 +1092,12 @@ namespace
       _oldMask = rpmlogSetMask( RPMLOG_UPTO( RPMLOG_PRI(RPMLOG_INFO) ) );
     }
 
-    ~RpmlogCapture()
-    {
+    RpmlogCapture(const RpmlogCapture &) = delete;
+    RpmlogCapture(RpmlogCapture &&) = delete;
+    RpmlogCapture &operator=(const RpmlogCapture &) = delete;
+    RpmlogCapture &operator=(RpmlogCapture &&) = delete;
+
+    ~RpmlogCapture() {
       rpmlogSetCallback( nullptr, nullptr );
       rpmlogSetMask( _oldMask );
     }
@@ -1242,7 +1221,7 @@ namespace
       }
 
       ++count[lineres];
-      detail_r.push_back( RpmDb::CheckPackageDetail::value_type( lineres, std::move(line) ) );
+      detail_r.push_back( RpmDb::CheckPackageDetail::value_type( lineres, line ) );
     }
 
     // Now combine the overall result:
@@ -1460,9 +1439,11 @@ RpmDb::run_rpm (const RpmArgVec& opts,
   p = copy (opts.begin (), opts.end (), p);
   *p = 0;
 
+#if 0 // if this is needed we need to forcefully close the db of running db_const_iterators
   // Invalidate all outstanding database handles in case
   // the database gets modified.
   librpmDb::dbRelease( true );
+#endif
 
   // Launch the program with default locale
   process = new ExternalProgram(argv, disp, false, -1, true);
@@ -1495,6 +1476,7 @@ bool RpmDb::systemReadLine( std::string & line )
 
           // we might have received a partial line, lets not forget about it
           line += readResult.second;
+          break;
         }
         case io::ReceiveUpToResult::Error:
         case io::ReceiveUpToResult::EndOfFile: {
@@ -1647,12 +1629,16 @@ void RpmDb::processConfigFiles(const std::string& line, const std::string& name,
 
 ///////////////////////////////////////////////////////////////////
 //
-//
 //	METHOD NAME : RpmDb::installPackage
-//	METHOD TYPE : PMError
 //
 void RpmDb::installPackage( const Pathname & filename, RpmInstFlags flags )
+{ installPackage( filename, flags, nullptr ); }
+
+void RpmDb::installPackage( const Pathname & filename, RpmInstFlags flags, RpmPostTransCollector* postTransCollector_r )
 {
+  if ( postTransCollector_r && postTransCollector_r->hasPosttransScript( filename ) )
+    flags |= rpm::RPMINST_NOPOSTTRANS;  // Just set the flag here. In \ref doInstallPackage we collect what else is needed.
+
   callback::SendReport<RpmInstallReport> report;
 
   report->start(filename);
@@ -1660,7 +1646,7 @@ void RpmDb::installPackage( const Pathname & filename, RpmInstFlags flags )
   do
     try
     {
-      doInstallPackage(filename, flags, report);
+      doInstallPackage( filename, flags, postTransCollector_r, report );
       report->finish();
       break;
     }
@@ -1681,7 +1667,7 @@ void RpmDb::installPackage( const Pathname & filename, RpmInstFlags flags )
   while (true);
 }
 
-void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, callback::SendReport<RpmInstallReport> & report )
+void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, RpmPostTransCollector* postTransCollector_r, callback::SendReport<RpmInstallReport> & report )
 {
   FAILIFNOTINITIALIZED;
   HistoryLog historylog;
@@ -1702,6 +1688,14 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
 
   // run rpm
   RpmArgVec opts;
+#if defined(WORKAROUNDDUMPPOSTTRANSBUG)
+  if ( postTransCollector_r && _root == "/" ) {
+#else
+  if ( postTransCollector_r ) {
+#endif
+    opts.push_back("--define");           // bsc#1041742: Attempt to delay %transfiletrigger(postun|in) execution iff rpm supports it.
+    opts.push_back("_dump_posttrans 1");  // Old rpm ignores the --define, new rpm injects 'dump_posttrans:' lines to collect and execute later.
+  }
   if (flags & RPMINST_NOUPGRADE)
     opts.push_back("-i");
   else
@@ -1753,16 +1747,21 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
   // LEGACY: collect and forward additional rpm output in finish
   std::string rpmmsg;
   std::vector<std::string> configwarnings;	// TODO: immediately process lines rather than collecting
+  std::vector<std::string> runposttrans;	// bsc#1041742: If rpm supports --runposttrans it injects 'dump_posttrans:' lines we do collect
 
   while ( systemReadLine( line ) )
   {
     if ( str::startsWith( line, "%%" ) )
     {
-      int percent;
+      int percent = 0;
       sscanf( line.c_str() + 2, "%d", &percent );
       report->progress( percent );
       continue;
     }
+    if ( str::hasPrefix( line, "dump_posttrans:" ) ) {
+      runposttrans.push_back( line );
+      continue;
+    }
     ++lineno;
     cmdout.set( "lineno", lineno );
     report->report( cmdout );
@@ -1781,6 +1780,10 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
     rpmmsg += "[truncated]\n";
 
   int rpm_status = systemStatus();
+  if ( postTransCollector_r && rpm_status == 0 ) {
+    // Before doing anything else, handle any pending %posttrans script or dump_posttrans lines.
+    postTransCollector_r->collectPosttransInfo( filename, runposttrans );
+  }
 
   // evaluate result
   for (std::vector<std::string>::iterator it = configwarnings.begin();
@@ -1810,7 +1813,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
     auto excpt { RpmSubprocessException(_("RPM failed: ") + error_message ) };
     if ( not rpmmsg.empty() )
       excpt.addHistory( rpmmsg );
-    ZYPP_THROW(std::move(excpt));
+    ZYPP_THROW(excpt);
   }
   else if ( ! rpmmsg.empty() )
   {
@@ -1829,26 +1832,23 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
 
 ///////////////////////////////////////////////////////////////////
 //
-//
 //	METHOD NAME : RpmDb::removePackage
-//	METHOD TYPE : PMError
 //
 void RpmDb::removePackage( Package::constPtr package, RpmInstFlags flags )
-{
-  // 'rpm -e' does not like epochs
-  return removePackage( package->name()
-                        + "-" + package->edition().version()
-                        + "-" + package->edition().release()
-                        + "." + package->arch().asString(), flags );
+{ removePackage( std::move(package), flags, nullptr ); }
+
+void RpmDb::removePackage( const std::string & name_r, RpmInstFlags flags )
+{ removePackage( name_r, flags, nullptr ); }
+
+void RpmDb::removePackage( const Package::constPtr& package, RpmInstFlags flags, RpmPostTransCollector* postTransCollector_r )
+{ // 'rpm -e' does not like epochs
+  removePackage( package->name()
+                 + "-" + package->edition().version()
+                 + "-" + package->edition().release()
+                 + "." + package->arch().asString(), flags, postTransCollector_r );
 }
 
-///////////////////////////////////////////////////////////////////
-//
-//
-//	METHOD NAME : RpmDb::removePackage
-//	METHOD TYPE : PMError
-//
-void RpmDb::removePackage( const std::string & name_r, RpmInstFlags flags )
+void RpmDb::removePackage( const std::string & name_r, RpmInstFlags flags, RpmPostTransCollector* postTransCollector_r )
 {
   callback::SendReport<RpmRemoveReport> report;
 
@@ -1857,7 +1857,7 @@ void RpmDb::removePackage( const std::string & name_r, RpmInstFlags flags )
   do
     try
     {
-      doRemovePackage(name_r, flags, report);
+      doRemovePackage( name_r, flags, postTransCollector_r, report );
       report->finish();
       break;
     }
@@ -1878,8 +1878,7 @@ void RpmDb::removePackage( const std::string & name_r, RpmInstFlags flags )
   while (true);
 }
 
-
-void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, callback::SendReport<RpmRemoveReport> & report )
+void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, RpmPostTransCollector* postTransCollector_r, callback::SendReport<RpmRemoveReport> & report )
 {
   FAILIFNOTINITIALIZED;
   HistoryLog historylog;
@@ -1904,6 +1903,14 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
 
   // run rpm
   RpmArgVec opts;
+#if defined(WORKAROUNDDUMPPOSTTRANSBUG)
+  if ( postTransCollector_r && _root == "/" ) {
+#else
+  if ( postTransCollector_r ) {
+#endif
+    opts.push_back("--define");           // bsc#1041742: Attempt to delay %transfiletrigger(postun|in) execution iff rpm supports it.
+    opts.push_back("_dump_posttrans 1");  // Old rpm ignores the --define, new rpm injects 'dump_posttrans:' lines to collect and execute later.
+  }
   opts.push_back("-e");
   opts.push_back("--allmatches");
 
@@ -1935,6 +1942,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
 
   // LEGACY: collect and forward additional rpm output in finish
   std::string rpmmsg;
+  std::vector<std::string> runposttrans;	// bsc#1041742: If rpm supports --runposttrans it injects 'dump_posttrans:' lines we do collect
 
   // got no progress from command, so we fake it:
   // 5  - command started
@@ -1943,6 +1951,10 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
   report->progress( 5 );
   while (systemReadLine(line))
   {
+    if ( str::hasPrefix( line, "dump_posttrans:" ) ) {
+      runposttrans.push_back( line );
+      continue;
+    }
     ++lineno;
     cmdout.set( "lineno", lineno );
     report->report( cmdout );
@@ -1957,6 +1969,11 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
     rpmmsg += "[truncated]\n";
   report->progress( 50 );
   int rpm_status = systemStatus();
+  if ( postTransCollector_r && rpm_status == 0 ) {
+    // Before doing anything else, handle any pending %posttrans script or dump_posttrans lines.
+    // 'remove' does not trigger %posttrans, but it may trigger %transfiletriggers.
+    postTransCollector_r->collectPosttransInfo( runposttrans );
+  }
 
   if ( rpm_status != 0 )
   {
@@ -1969,7 +1986,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
     auto excpt { RpmSubprocessException(_("RPM failed: ") + error_message ) };
     if ( not rpmmsg.empty() )
       excpt.addHistory( rpmmsg );
-    ZYPP_THROW(std::move(excpt));
+    ZYPP_THROW(excpt);
   }
   else if ( ! rpmmsg.empty() )
   {
@@ -1986,6 +2003,62 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
   }
 }
 
+///////////////////////////////////////////////////////////////////
+//
+//	METHOD NAME : RpmDb::runposttrans
+//
+int RpmDb::runposttrans( const Pathname & filename_r, const std::function<void(const std::string&)>& output_r )
+{
+  FAILIFNOTINITIALIZED;
+  HistoryLog historylog;
+
+  MIL << "RpmDb::runposttrans(" << filename_r << ")" << endl;
+
+  RpmArgVec opts;
+  opts.push_back("-vv");  // want vverbose output to see scriptlet execution in the log
+  opts.push_back("--runposttrans");
+  opts.push_back(filename_r.c_str());
+  run_rpm (opts, ExternalProgram::Stderr_To_Stdout);
+
+  // Tailored to suit RpmPostTransCollector.
+  // It's a pity, but we need all those verbose debug lines just
+  // to figure out which script is currently executed. Otherwise we
+  // can't tell which output belongs to which script.
+  static const str::regex rx( "^D: (%.*): scriptlet start$" );
+  str::smatch what;
+  std::string line;
+  bool silent = true; // discard everything before 1st scriptlet
+  while ( systemReadLine(line) )
+  {
+    if ( not output_r )
+      continue;
+
+    if ( str::startsWith( line, "D:" ) ) {  // rpm debug output
+      if ( str::regex_match( line, what, rx ) ) {
+        // forward ripoff header
+        output_r( "RIPOFF:"+what[1] );
+        if ( silent )
+          silent = false;
+      }
+      continue;
+    }
+    if ( silent ) {
+      continue;
+    }
+    if ( str::startsWith( line, "+ " ) ) {  // shell -x debug output
+      continue;
+    }
+    // forward output line
+    output_r( line );
+  }
+
+  int rpm_status = systemStatus();
+  if ( rpm_status != 0 ) {
+    WAR << "rpm --runposttrans returned " << rpm_status << endl;
+  }
+  return rpm_status;
+}
+
 ///////////////////////////////////////////////////////////////////
 //
 //
